/* $Id: MakeInharmonicity.java,v 1.2 2016/08/29 23:19:03 mit3 Exp $ */
/* MakeInharmonicity.java v0.1.1 */

public class MakeInharmonicity {
    private final int KEY_BOARD = 88;
    private final int OCTAVE = 12;
    private final int CURVE = 0;
    private final int STRHT = 1;
    private final int LOGG = 2;

    private boolean isCalc;
    private int ITYPE = CURVE;
    private int WOUND_WIRE = 28;
    private double GRADE_BASS = 0.06;
    private double GRADE_TREBLE = 0.087;
    private double A49I = 0.55;
    private double[] Inha;

    public MakeInharmonicity() {
        Inha = new double[KEY_BOARD];
    }
    public void setBassGrade(double bg) {
	GRADE_BASS = bg;
    }
    public void setTrebleGrade(double tg) {
	GRADE_TREBLE = tg;
    }
    public void setA49i(double a49i) {
	A49I = a49i;
    }
    public void setWoundWire(int ww) {
	WOUND_WIRE = ww;
    }
    public void setIType(int num) {
	if (num >= 0 && num < 3) 
	    ITYPE = num;
	else
	    System.out.println("*** Inha.type error.:"+num+" ***"); 
    }
    public double[] getInha() {
	//Inha = new double[KEY_BOARD];
	double sft = makeTrebleInha();

	switch(ITYPE) {
	case CURVE:
	    makeBassInha(sft);
	    break;
	case STRHT:
	    makeBassStraight(sft);
	    break;
	case LOGG:
	    makeBassLog(sft);
	    break;
	default:
	    makeBassInha(sft);
	    break;
	}
	isCalc = true;
	return Inha;
    }
    public double getInha(int num) {
	if (isCalc)
	    return Inha[num];
	else {
	    System.out.println("*** getInha error.:"+num+" ***");

	    return 0;
	}
    }
    public void resetIsCalc() {
	isCalc = false;
    }
    public boolean isCalc() {
	return isCalc;
    }
    public int getWoundWire() {
	return WOUND_WIRE;
    }
    public double makeTrebleInha() {
        for (int i = WOUND_WIRE; i < KEY_BOARD; i++) {
	    Inha[i] = Math.exp(i*GRADE_TREBLE);
        }

	double df = A49I/Inha[48];
	for (int i = WOUND_WIRE; i < KEY_BOARD; i++) {
	    Inha[i] *= df;
	}

	return Inha[WOUND_WIRE]; // Ǿ᤹ͤ
    }
    public void makeBassStraight(double sft) {
	double[] inha = new double[WOUND_WIRE+1];

	for (int i = 0; i < WOUND_WIRE+1; i++) {
	    inha[i] = 1.0/Math.exp(i*GRADE_BASS*0.8);
        }

	addInha(inha, sft);
    }
    public void makeBassLog(double sft) {
	double[] inha = new double[WOUND_WIRE+1];

	for (int i = 0; i < WOUND_WIRE+1; i++) {
	    inha[i] = Math.pow(i+1, -GRADE_BASS*6.5);
	}

	addInha(inha, sft);
    }
    public void addInha(double[] inha, double sft) {
	if (WOUND_WIRE < 1)
	    return;

	double df = inha[WOUND_WIRE]/sft;

	for (int i = 0; i < WOUND_WIRE; i++) {
	    Inha[i] = inha[i]/df;
	}
    }
    public void makeBassInha(double sft) { // curve
	double[] inha = new double[WOUND_WIRE+1];

	for (int i = 0; i < WOUND_WIRE+1; i++) {
	    inha[i] = cosh(GRADE_BASS*(i-WOUND_WIRE));
	}

	addInha(inha, sft);
    }
    public double cosh(double x) {
        double me = Math.pow(Math.E, x);

        return (me+1/me)/2;
    }
    //
    public String toString() {
        StringBuffer sb = new StringBuffer();

        sb.append("GradeBass:"+(float)GRADE_BASS);
	sb.append(" GradeTreble:"+(float)GRADE_TREBLE);
        sb.append(" Pitch:"+(float)A49I);
        sb.append(" WoundWire:"+WOUND_WIRE);

	return sb.toString();
    }
}