/* $Id: TuningTriads.java,v 1.4 2017/02/15 07:57:41 mit3 Exp $ */
/* TuningTriads v0.2 */

public class TuningTriads {
    private double[] Cent;
    private double[] Inha;
    private int[] OCT;
    private int[] P5TH;
    private int[] P4TH;
    private Interval I;
    private final int fromKey = 32-1;
    private final int toKey = 44-1;
    private final int P5 = 7;
    private final int P4 = 5;
    private final int KB = 88;
    private final int OCTAVE = 12;
    private final double REVOCT = 1.0/OCTAVE;
    private final double PITCH = 440; // ԥå

    public TuningTriads() {
	I = new Interval();
	OCT = I.getIdata(OCTAVE); // 2,1,12
	P5TH = I.getIdata(P5); // 3,2,7
	P4TH = I.getIdata(P4); // 4,3,5
    }
    public void setInha(double[] inha) {
	Inha = inha;
    }
    public void setCent(double[] cent) {
	Cent = cent;
    }
    public double[] getCent() {
	return Cent;
    }
    public double getCent(int key) {
	return Cent[key];
    }
    public void Tuning() {
	tuneLaying();
	tuneBass();
	tuneTreble();
    }
    //
    public void tuneLaying() {
	double sa = 1.9;
	double df = sa / OCTAVE;
	double sf = 4 * df;
	double cnt = 0;
	double val;

	for (int i = fromKey; i < toKey; i++) {
	    val = cnt * df - sf;
	    Cent[i] = val;
	    cnt += 1;
	}
    }
    public void tuneBass() {
	for (int i = fromKey-1; i >= 0; i--) {
	    tune_bass(i);
	}
    }
    public void tune_bass(int key) {
	int keyh0 = key + OCT[2];
	int keyh5 = key + P5TH[2];
	int keyh4 = key + P4TH[2];
	double dc = -0.1;
	double minc = 0;
	double minb = 10000;
	double dif;
	double minx = (Inha[key] > 3.0) ? minb : 1.1; // ϰϰʾ
	double maxc = (Inha[key] > 3.0) ? -100 : -60; // "

	while (true) {
	    dif = getTriadsBass(key, keyh0, keyh5, keyh4);
 	   
	    if (dif < minx) {
		minb = dif;
		minc = Cent[key];
	    }
	    if (Cent[key] > -maxc)
		break;

	    Cent[key] -= dc;
	}
	while (true) {
	    dif = getTriadsBass(key, keyh0, keyh5, keyh4);

	    if (dif < minx) {
		minb = dif;
		minc = Cent[key];
	    }
	    if (Cent[key] < maxc)
		break;

	    Cent[key] += dc;
	}

	Cent[key] = minc;
    }
    public double getTriadsBass(int key, int ko, int k5, int k4) {
	return Math.abs( get_ibeats(ko, key, OCT)+
			 get_ibeats(k5, key, P5TH)+
			 get_ibeats(k4, key, P4TH) );
    }
    public double get_ibeats(int keyh, int keyl, int[] ratio) {
	double fh = get_ifreq(keyh, ratio[1]);
	double fl = get_ifreq(keyl, ratio[0]);

	return (fh - fl);
    }
    public double get_ifreq(int key, int par) {
	return c2f(getFreq(key)*par, get_icent(key, par));
    }
    public double c2f(double f, double c) {
	return f * Math.pow(2, c / 1200);
    }
    public double getFreq(int key) { // 0~87
	return PITCH * Math.pow(2, (key - 48)*REVOCT);
    }
    public double get_icent(int key, int par) {
	return Cent[key] + calcInha(key, par);
    }
    public double calcInha(int key, int par) {
	return Inha[key] * par * par;
    }
    //
    public void tuneTreble() {
	for (int i = toKey; i < KB; i++) {
	    tune_treble(i);
	}
    }
    public void tune_treble(int key) {
	int keyl0 = key - OCT[2];
	int keyl5 = key - P5TH[2];
	int keyl4 = key - P4TH[2];
	int max = 60;
	double dc = 0.1;
	double limit = 1.1;
	double minc = 0;
	double minb = 10000;
	double dif;

	while (true) {
	    dif = getTriadsTreble(key, keyl0, keyl5, keyl4);

	    if (dif < limit) {
		minb = dif;
		minc = Cent[key];
	    }
	    if (Cent[key] < -max)
		break;
	    Cent[key] -= dc;

	}
	while (true) {
	    dif = getTriadsTreble(key, keyl0, keyl5, keyl4);

	    if (dif < limit) {
		minb = dif;
		minc = Cent[key];
	    }
	    if (Cent[key] > max)
		break;
	    Cent[key] += dc;
	}
	
	Cent[key] = minc;
    }
    public double getTriadsTreble(int key, int ko, int k5, int k4) {
	return Math.abs( get_ibeats(key, ko, OCT)+
			 get_ibeats(key, k5, P5TH)+
			 get_ibeats(key, k4, P4TH) );
    }
}
